# -*- coding: utf-8 -*-
#
# Copyright Grégory Soutadé 2015

# This file is part of iwla

# iwla is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# iwla is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with iwla.  If not, see <http://www.gnu.org/licenses/>.
#

from iwla import IWLA
from iplugin import IPlugin
from display import *

"""
Display hook

Create TOP hits page

Plugin requirements :
    post_analysis/top_hits

Conf values needed :
    max_hits_displayed*
    create_all_hits_page*

Output files :
    OUTPUT_ROOT/year/month/top_hits.html
    OUTPUT_ROOT/year/month/index.html

Statistics creation :
    None

Statistics update :
    None

Statistics deletion :
    None
"""

class IWLADisplayTopHits(IPlugin):
    def __init__(self, iwla):
        super(IWLADisplayTopHits, self).__init__(iwla)
        self.API_VERSION = 1
        self.requires = ['IWLAPostAnalysisTopHits']
        self.max_hits = self.iwla.getConfValue('max_hits_displayed', 0)
        self.create_all_hits = self.iwla.getConfValue('create_all_hits_page', True)

    def hook(self):
        display = self.iwla.getDisplay()
        top_hits = self.iwla.getMonthStats()['top_hits']
        top_hits = sorted(top_hits.items(), key=lambda t: t[1], reverse=True)

        # All in a file
        if self.create_all_hits:        
            title = createCurTitle(self.iwla, u'All Hits')
            filename = 'top_hits.html'
            path = self.iwla.getCurDisplayPath(filename)

            page = display.createPage(title, path, self.iwla.getConfValue('css_path', []))
            table = display.createBlock(DisplayHTMLBlockTable, self.iwla._(u'All Hits'), [self.iwla._(u'URI'), self.iwla._(u'Entrance')])        
            table.setColsCSSClass(['', 'iwla_hit'])
            total_hits = [0]*2
            new_list = self.max_hits and top_hits[:self.max_hits] or top_hits
            for (uri, entrance) in new_list:
                table.appendRow([generateHTMLLink(uri), entrance])
                total_hits[1] += entrance
            if self.max_hits:
                others = 0
                for (uri, entrance) in top_hits[self.max_hits:]:
                    others += entrance
                table.appendRow([self.iwla._(u'Others'), others])
                table.setCellCSSClass(table.getNbRows()-1, 0, 'iwla_others')
                
            page.appendBlock(table)
                        
            display.addPage(page)

        title = u'Top Hits'
        if self.create_all_hits:
            link = '<a href=\'%s\'>%s</a>' % (filename, self.iwla._(u'All Hits'))
            title = '%s - %s' % (title, link)

        # Top in index
        index = self.iwla.getDisplayIndex()
        
        table = display.createBlock(DisplayHTMLBlockTable, title, [self.iwla._(u'URI'), self.iwla._(u'Entrance')])        
        table.setColsCSSClass(['', 'iwla_hit'])
        for (uri, entrance) in top_hits[:10]:
            table.appendRow([generateHTMLLink(uri), entrance])
            total_hits[1] -= entrance
        if total_hits[1]:
            total_hits[0] = self.iwla._(u'Others')
            table.appendRow(total_hits)
            table.setCellCSSClass(table.getNbRows()-1, 0, 'iwla_others')
        index.appendBlock(table)
